/** 
 *  @file   FrameQBufMgr.xdc
 *
 *  @brief
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */





package ti.syslink.ipc.rtos;

import xdc.runtime.Error;
import xdc.runtime.IGateProvider;
import ti.sdo.utils.NameServer;

@ModuleStartup
@InstanceInitError /* Initialization may throw errors */
@InstanceFinalize
module FrameQBufMgr inherits ti.syslink.ipc.rtos.interfaces.IFrameQBufMgr {

    /*!
     *  Assert raised when an invalid parameter(s) is passed into a function
     */
    config xdc.runtime.Assert.Id A_invalidParams =
        {msg: "A_invalidParams: Invalid parameter"};

    /*!
     *  Assert raised when a null pointer needs to be dereferenced
     */
    config xdc.runtime.Assert.Id A_nullPointer =
        {msg: "A_nullPointer: Null pointer"};

   /*!
     *  Assert raised when null handle is returned from  modules.
     */
    config xdc.runtime.Assert.Id A_nullHandle =
        {msg: "A_nullHandle: Null Handle"};
   /*!
     *  Assert raised when Inst No of the instance is not valid.
     */
    config xdc.runtime.Assert.Id A_InvalidInstNo  = {
        msg: "A_InvalidInstNo: Invalid instance no"
    };

   /*!
     *  Assert raised when creator processor id of the instance is not valid.
     */
    config xdc.runtime.Assert.Id A_InvalidCreatorProcId  = {
        msg: "A_InvalidCreatorProcId: Invalid creator processor id"
    };

    /*!
     *  Error raised when an invalid parameter(s) is passed into a function
     */
    config Error.Id E_InvalidParams =
        {msg: "E_InvalidParams: Invalid parameter"};

    /*!
     *  Error raised when an invalid interface type mentioned in params
     */
    config Error.Id E_InvalidInterfaceType =
        {msg: "E_InvalidInterfaceType: Invalid interface type"};


    /*!
     *  Error raised when an instance already exists
     */
    config Error.Id E_InstanceExists = {
                msg: "E_InstanceExists: Instance already exists"
        };

    /*!
     *  Error raised when FrameQBufMgr is not created.
     */
    config Error.Id E_FrameQBufMgrNotCreated  = {
        msg: "E_FrameQBufMgrNotCreated: FrameQBufMgr instance is not created"
    };

    /*!
     *  Error raised when  internal FrameQBufMgr create failed.
     */
    config Error.Id E_FrameQBufMgrCreateFail  = {
        msg: "E_FrameQBufMgrCreateFail: Internal FrameQBufMgr instance create API failed"
    };

    /*!
     *  Error raised when FrameQBufMgr open failed.
     */
    config Error.Id E_FrameQBufMgrOpenFail  = {
        msg: "E_FrameQBufMgrOpenFail: FrameQBufMgr instance open failed"
    };

    /*!
     *  Error raised when FrameQBufMgr is not Found in NameServer.
     */
    config Error.Id E_FrameQBufMgrNotFound  = {
        msg: "E_FrameQBufMgrNotFound: FrameQBufMgr instance is not found in NameServer"
    };

    /*!
     *  Error raised when ClientNotifyMgr Create failed.
     */
    config Error.Id E_ClientNotifyMgrCreate  = {
        msg: "E_ClientNotifyMgrCreate: ClientNotifyMgr instance Create Failed"
    };

    /*!
     *  Error raised when ClientNotifyMgrOpen failed.
     */
    config Error.Id E_ClientNotifyMgrOpen  = {
        msg: "E_ClientNotifyMgrOpen: ClientNotifyMgrOpen instance Open Failed"
    };

    /*!
     *  Error raised when Insufficient shared memory provided to create call.
     */
    config Error.Id E_InvSize  = {
        msg: "E_InvSize: SharedMemory provided to create call is less than the instance shared memory requirements"
    };

    /*!
     *  Error raised when Insufficient shared addrprovided to create call.
     */
    config Error.Id E_InvAddr  = {
        msg: "E_InvAddr: Shared Addr provide to create call is NULL"
    };
    /*!
     *  Error raised when buffersize is not aligned to Cache line size.
     */
    config Error.Id E_CacheAlign  = {
        msg: "E_CacheAlign: BufferSize is not Cache Aligned"
    };
    /*!
     *  Error raised when max instances limit is reached
     */
    config Error.Id E_maxInstances  = {
        msg: "E_maxInstances: max instances limit reached"
    };

    /*! 
     *  ======== maxRuntimeEntries ========
     *  Maximum number of FrameQBUfMgr's that can be dynamically created
     *  and added to the NameServer.
     */
    metaonly config UInt maxRuntimeEntries = NameServer.ALLOWGROWTH; 

    /*! @_nodoc
     *  ======== OpenParams ========
     *  mouduleconfig structure.
     *
     *  @field(commonOpenParams)
         *                            Open params common to all the implementations.
     *
     */
        struct OpenParams {
            CommonOpenParams commonOpenParams;
        };

     /*!
      *  ========= NameServerEntryParams ==========
      *  Structure defining Additional specific parameters that name server maintains
      * alonh with the interface  type.
      *
      *  @field(sharedAddr) Virtual shared Region Address  for control structure.
      *  @field(sharedAddrSize) Size of the shared memory  for control structure.
      *  @field(instGateMPAddr)     Instance gateMP shared address in
	  *                             SharedRegion_SRPtr format.Valid only if it 
	  *                             is  created internally.
      *  @field(memMgrType_HdrBufs) MemMgr type  configured for Header buffers.
	  *  @field(memMgrType_FrmBufs) MemMgr type  configured for Frame  buffers.
	  *  @field(allocaterName_HdrBufs) Name of the HeapMemMp instance to be used 
	  *                                 Header buffer allocation.
	  *                                 TODO: need to check if it is valid for 
	  *                                   Tiler
	  *  @field(allocaterName_FrmBufs) Name of the HeapMemMp instance to be used 
	  *                                 frame buffer allocation.
	  *                                 TODO: need to check if it is valid for 
	  *                                   Tiler
      */
        struct ShMem_InstNameServerEntryParams  {
	        Ptr        sharedAddr;
	        UInt32     sharedAddrSize;
            Ptr        instGateMPAddr;
			UInt8      frmBuf_regionId;
			UInt8      hdrBuf_regionId;
			Ptr        cliGateMPAddr;
        }

    /*! @_nodoc
     *  ======== NameServerEntry ========
     *  mouduleconfig structure.
     *
     *  @field(interfaceType)  type of the interface. see {@link Interface}.
     *  @field(instParams)  instparams  is  union of the implementation's
         *                      nameserver entry.
         *                      the individual nameserver entry contents may vary
         *                      based on implementation.
     */
        struct NameServerEntry {
                UInt32     ctrlInterfaceType;
                UInt32     headerInterfaceType;
                UInt32     bufInterfaceType;
                union {
		            ShMem_InstNameServerEntryParams  shMemParams;
                } instParams;
        }

    /*! @_nodoc
     *  ======== isSupportedInterface ========
     *  API to find out  if the given interface type is supported.
     *
     *  @param(interfaceType)  type of the interface.
     *
     */
         Int32 isSupportedInterface(UInt32 interfaceType);

    /*! @_nodoc
     *  ======== isSupportedBufInterface ========
     *  API to find out  if the given interface type is supported for frame 
     *  buffers.
     *
     *  @param(ctrlInterfaceType)  type of the control interface.
     *  @param(bufInterfaceType)  type of the buf interface.
     */
        Int32 isSupportedBufInterface(UInt32 ctrlInterfaceType,
                                      UInt32 bufInterfaceType);

    /*!
     *  ======== open ========
     *  API to open the created instance and to get a handle to it.
     *
     *  The instance must be created usng create call before opening it.
     *
     *  @param(handlePtr)  Handle to the instance will be returned through this pointer.
     *  @param(openParams) open parameters specific  implementation.
     *
     */

/*    Int32  open(Handle *handlePtr,Ptr openParams);*/

    /*!
     *  ======== close ========
     *  Function to close the dynamically opened instance using {@link #open} call.
     *
     *
     *  @b(returns)                 returns 0 if  successfully registered the call back function.
     *
     */
/*     Int32 close(Handle * handlePtr); */

    /*!
     *  ======== getBaseHeaderSize ========
     *  Retruns the base header size of frame based on the number of frame buffers
     *  that will be contained in frame.
     *
     * @param(numFrameBufs)  Number of frame buffers in  frame.
     *  @b(returns)       Base frame header size in bytes.
     */
    UInt32 getBaseHeaderSize(UInt8 numFrameBufs);

    /*! @_nodoc
     *  ======== getInstNo ========
     *  API to get the unique instane no for the instances of FrmaeQBufMgr
     *  instances.
     *
     *  @param(instNo)  Location to receive the instance no.
     *  @b(returns)       Base frame header size in bytes.
     */
    Int32 getInstNo(UInt8 *instNo);

    /*! @_nodoc
     *  ======== getInstNo ========
     *  API to get the unique instane no for the instances of FrmaeQBufMgr
     *  instances.
     *
     *  @param(instNo)  Instance no that needs to be freed to the module.So that
	 *                  module can  allocate this instNo to other getInstNo call
	 *                  s.
     *  @b(returns)     Success (>=0) if successfully freed the instNo.
     */
    Int32 freeInstNo(UInt8 instNo);

    instance:

     config CreateParams commonCreateParams = {size:0,ctrlInterfaceType:INTERFACE_NONE,name:null,openFlag:false};
    /*!
     *  ======== impParams ========
     *  Pointer to implementation specific parameters
     *
     */
            config Ptr impParams = null;
    /*!
     *  ======== impParams ========
     *  Pointer to implementation specific parameters
     *
     */
     config UInt32  impParamsSize = 0;
    internal:

    Void postInit(Object *obj,const Params* params ) ;
    /*!
     *  ======== Module_State ========
     *  Module  state Object structure.
     *
     *  @field(nameServer)        Handle to the created NameServer instance for this HeapNotifyMutiBufSM module.
     *  @field(nameServerName)    Name of the NameServer instance
     *  @field(isInstNoValid)     Create call will check this flags and sets first invalid index to valid
     *                            and  assigns this index to the first byte of obj->Id as instNo.
     *
     */

    struct Module_State {
        NameServer.Handle     nameServer;
        UInt32                nameServerType;
        IGateProvider.Handle  gate;
        UInt32                gateType;
        Config                cfg;
        UInt32                refCount;
        Bool                  isInstNoValid[];
    };

    /*!
     *  ======== Instance_State ========
     *  Instance  state Object structure.
     *
     *  @field(gate)             Gate for critical region management.
     *  @field(exactMatchReq)    Flag indicating whether this client requires exact sized buffer or not.
     *  @field(isRegistered)     Flag to indicate whether this client has registed a notification function
     *                           with ClientNotifyMgr.
     *  @filed(Id)               Id of the instance.
     *  @field(notifyId)         Varible  to hold the  index returned by ClientNotifyMgr_registerClient.
     *  @filed(objType)          Indicates type of object. Dynamic create or Dynamic open etc
     *  @field(controlStruct)    Pointer to the shared instance control structure.
     *  @field(clientNotifyMgrHandle) Handle to the ClientNotifyMgr instance created for this FrameQBufMgr instance.
     *  @field(nameServerName)   Name of the NameServer instance .
     *
     */
    /* not for client use */
     struct Instance_State {
                 // IFrameQBufMgr.Handle    handle;
          Ptr                     handle;
          UInt32                  instId ;
          UInt32                  notifyId ;
          UInt32                  objType ;
          UInt32                  ctrlInterfaceType;
          UInt32                  headerInterfaceType;
          UInt32                  bufInterfaceType;
          UInt32                  refCount;
          Char                    name[32];
          Bool                    ctrlStructCacheFlag ;
          Bool                    frmHdrBufCacheFlag ;
          Bool                    frmBufCacheFlag[8] ;
          Bool                    bufCpuAccessFlag[8];
          Bool                    isRegistered ;
     };
}
