/*
 *  @file   SyslinkMemMgr.c
 *
 *  @brief   Implements front end for the SyslinkMemMgr implementations.
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */


#include <xdc/std.h>
#include <xdc/runtime/Assert.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/Memory.h>

#include <string.h>

#include <ti/syslink/inc/SyslinkMemMgr_errBase.h>
#include <ti/syslink/ipc/rtos/interfaces/ISyslinkMemMgr.h>


#include "package/internal/SyslinkMemMgr.xdc.h"


#if defined (__cplusplus)
extern "C" {
#endif

Int SyslinkMemMgr_Instance_init(SyslinkMemMgr_Object       *obj,
                                const SyslinkMemMgr_Params *params,
                                Error_Block                *eb)
{

    Int32 status = MEMMGR_SUCCESS;

    SyslinkMemMgr_RemoteProxyShMem_Params     custom1Params;
    SyslinkMemMgr_RemoteProxyTilerMem_Params  custom2Params;

    Error_init(eb);
    switch(params->memMgrType) {

        case SyslinkMemMgr_TYPE_SHAREDMEM:
            SyslinkMemMgr_RemoteProxyShMem_Params_init(&custom1Params);
            custom1Params.heapHandle = params->heapHandle;
            obj->memMgrHandle = SyslinkMemMgr_RemoteProxyShMem_create(&custom1Params, eb);
            obj->memMgrType = params->memMgrType;
        break;
        case SyslinkMemMgr_TYPE_TILERMEM:
            SyslinkMemMgr_RemoteProxyTilerMem_Params_init(&custom2Params);
            custom2Params.heapHandle = params->heapHandle;
            obj->memMgrHandle = SyslinkMemMgr_RemoteProxyTilerMem_create(&custom2Params, eb);
            obj->memMgrType = params->memMgrType;
        break;
        default:
        break;

    }
    return(status);
}

Void SyslinkMemMgr_Instance_finalize(SyslinkMemMgr_Object *obj,
                                     Int                   status)
{

    switch(obj->memMgrType) {

        case SyslinkMemMgr_TYPE_SHAREDMEM:
             SyslinkMemMgr_RemoteProxyShMem_delete(&obj->memMgrHandle);
        break;
        case SyslinkMemMgr_TYPE_TILERMEM:
             SyslinkMemMgr_RemoteProxyTilerMem_delete(&obj->memMgrHandle);
        break;
        default:
        break;

    }

}


/* Function to allocate memory from the SyslinkMemMgr */
Ptr
SyslinkMemMgr_alloc (SyslinkMemMgr_Object *obj,
                     Ptr                   allocParams)
{

    //Assert_isTrue(obj != NULL, SyslinkMemMgr_A_nullArgument);

    return (ISyslinkMemMgr_alloc(obj->memMgrHandle, allocParams));
}


/* Function to allocate memory from the SyslinkMemMgr */
Int32
SyslinkMemMgr_free (SyslinkMemMgr_Object *obj,
                    Ptr                  freeParams)
{
    //Assert_isTrue(obj != NULL, SyslinkMemMgr_A_nullArgument);

    return (ISyslinkMemMgr_free(obj->memMgrHandle, freeParams));

}
#if 0
/* Function to map. not used for shared memory*/
Ptr
SyslinkMemMgr_map (SyslinkMemMgr_Object *obj,
                   Ptr                   arg)
{
    Ptr   ptr = NULL;

    //Assert_isTrue(obj != NULL, SyslinkMemMgr_A_nullArgument);

    return (ISyslinkMemMgr_map(obj->memMgrHandle, arg));
}

/* Function to do unmap */
Int32
SyslinkMemMgr_unmap (SyslinkMemMgr_Object *obj,
                     Ptr                   arg)
{
    Int32 status = MEMMGR_SUCCESS;

    //Assert_isTrue(obj != NULL, SyslinkMemMgr_A_nullArgument);

    return (ISyslinkMemMgr_unmap(obj->memMgrHandle, arg));
}
#endif
/* Function to translate source address to destination address type */
Ptr
SyslinkMemMgr_translate (SyslinkMemMgr_Object   *obj,
                         Ptr                     srcAddr,
                         SyslinkMemMgr_AddrType  srcAddrType,
                         SyslinkMemMgr_AddrType  desAddrType)
{

    //Assert_isTrue(obj != NULL, SyslinkMemMgr_A_nullArgument);

    return (ISyslinkMemMgr_translate(obj->memMgrHandle,
                                     srcAddr,
                                     srcAddrType,
                                     desAddrType));
}

#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
